/*

 A Sample WinLog device driver .dll
 Created by Justin Luton - 1/4/07 DevTechnics (http://www.devtechnics.com)

PUBLIC SOFTWARE 
We have placed this source code in the public domain. You can use, modify, and distribute
the source code and executable programs based on the source code. 

DISCLAIMER OF WARRANTY 
This source code is provided "as is" and without warranties as to performance or merchantability.
The author and/or distributors of this source code may have made statements about this source code.
Any such statements do not constitute warranties and shall not be relied on by the user in deciding
whether to use this source code. 

This source code is provided without any express or implied warranties whatsoever. Because of the
diversity of conditions and hardware under which this source code may be used, no warranty of fitness
for a particular purpose is offered. The user is advised to test the source code thoroughly before relying
on it. The user must assume the entire risk of using the source code. 

*/


#include "stdafx.h"
#include "sample.h"
#include "stdio.h"
#include "configdlg.h"
#include "stdlib.h"
#include "math.h"

HANDLE hInst = NULL;

// A variable to store our configuration parameters in, normally,
// you won't need to change this declaration! The configuration
// structure is defined in sample.h
SAMPLE_CONFIG config;

// A structure to store our field variables, change this structure
// to include the list of your devices fields, these field types
// should be only float, int, or char.
typedef struct
{
   float samplefloat;
   int sampleint;
   char samplestring[32];
} FIELDS;

// And the declaration of our variables, normally, you won't need to
// change this!
FIELDS fields;

// The number of fields that we'll be sending back to WinLog, this is normally a count
// of the variables defined in the "FIELDS" structure above. Change this to match
// the number of fields defined above.
#define NUM_FIELDS 3

// The display names for each of the fields that will be sent back and displayed to the user.
// Change this to describe your fields
char *fieldnames[]=
{
	"sample floating point field",
	"sample integer field",
	"sample text field",
};

// the format of each of the fields that will be sent back to WinLog:
// 'f' - floating point field 
// 'd' - integer field
// 's' - string field
// Change this to describe the data type for each of your fields.
char *fieldformats = "fds";

// This is simply an optimization so that the driver can quickly lookup a field when WinLog requests
// it, you'll see below that we set pointers to the fields defined in the fields structure above when
// we initialize below.
void *fielddata[NUM_FIELDS];

// prototype for the callback to Winlog, initialized below in the Initialize function, you shouldn't
// need to change this!
int (__cdecl *notifycallback)(unsigned char notification, void *hint, void *hint2) = NULL;
void *notifyhint = NULL;


// Some object handles to notify our driver thread of events
HANDLE m_hThreadTerm = NULL;
HANDLE m_hThreadStarted = NULL;
HANDLE m_hThreadStoppedEvent = NULL;


// The main entry point to our driver
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{

	hInst = hModule;
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		{
			// Set your driver defaults here...
			memset(&fields, 0, sizeof(FIELDS));
			config.mysamplesetting1 = 0;
			config.mysamplesetting1 = 1;
					}
		break;
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

// SetConfig
// This API is called by Winlog to notify the driver of it's configuration, no actual 
// reconfiguration of the device should be done here, instead, just copy the configuration
// to your local structure. Usually, this code won't need to be changed.
SAMPLE_API void SetConfig(void *pConfig)
{
	memcpy(&config, pConfig, sizeof(SAMPLE_CONFIG));
}

// GetConfig
// This API is called by Winlog to get the current configuration of the driver.
// Return a pointer to the configuration data.
// Usually, this code won't need to be changed.
SAMPLE_API void *GetConfig(void)
{
	return (void*)&config;
}

// GetConfigSize
// This API is called by winlog to determine the size of the configuration data that the
// driver needs to be stored.
// Return the size of the configuration data in bytes.
// Usually, this code won't need to be changed.
SAMPLE_API int GetConfigSize(void)
{
	return sizeof(SAMPLE_CONFIG);
}

// This is the main thread procedure for our driver, you'll want to do the specifics
// of communucating with your device in here!
// When fields are available to be logged, call the notifycallback function as below!
DWORD WINAPI ThreadProc(LPVOID lpParameter)
{
	char workstring[255];
    SetEvent(m_hThreadStarted);

	// We'll just generate a new sample every 100ms... Normally, you'll do something
	// like check your COM port or other communications port here, and process the
	// received data.
	while (WaitForSingleObject(m_hThreadTerm, 100) != WAIT_OBJECT_0)
	{
		if (notifycallback)
		{
			// Fabricate some dummy data, and send it back to Winlog!
			fields.samplefloat += 0.01;
			if (fields.samplefloat >= 100)
				fields.samplefloat = 0;

			fields.sampleint = (rand() + (int)notifyhint) % 100;
			sprintf(workstring, "test%d", fields.sampleint);
			strcpy(fields.samplestring, workstring);
			
			// Here's where we notify WinLog of data being ready!
			notifycallback(0, notifyhint, 0);
		}
	}

	if (m_hThreadStoppedEvent)
		SetEvent(m_hThreadStoppedEvent);


	return 0;
}

/*
// Initialize
// This function is called by winlog to initialize the driver, communication with the device
// should be attempted here. Also, the device can be configured here, since it is guaranteed
// that SetConfig will be called prior to this, so your configuration data will be valid! 
// The function should return 1 if the device was sucessfully initialized, otherwise, return
// 0.

// The following parameters are passed to initialize:
HWND parent - The parent window that is initializing the driver
void *ptr - A pointer to the callback function that you can call to notify winlog of driver
            events. (See below for notification values)
void *hint - A value that should always be passed back as the NotifyCallback hint parameter.


NotifyCallback spec:
The notifycallback function informs WinLog of driver events, The parameters to the function
are:

NotifyCallback (unsigned char notification, void *hint, void *hint2);

unsigned char notification - This is the type of notification that you are informing WinLog
of, it can be one of the following:

// A notify callback value of 0 represents a sample is ready to be retrieved...
	notifycallback(0, notifyhint, 0);


// A notify callback value of 1 requests the front end to open a log file for us to dump data to, without
// updating gauges during the download. This is usually done to download stored log data to a file. If there
// is a failure to open the logfile, the function will return 0, otherwise, non-zero
if (!notifycallback(1, notifyhint, 0))
{
	return 0;
}

// A notify callback value of 2 requests the front end to close the currently open log file.
notifycallback(2, notifyhint, 0);

// a notify callback value of 3 will send a debug message to winlog.
if (notifycallback)
	notifycallback(3, notifyhint, msg);

void *hint - This value should always be the "hint" parameter that was passed in to the
             Initialize function.

void *hint2 - Notification specific. For example, the "Debug Message" notification uses
              this parameter to pass a Debug String to winlog.

*/

SAMPLE_API int Initialize(HWND parent, void *ptr, void *hint)
{
	DWORD threadid;
	HANDLE hThread;
	notifyhint = hint;
	notifycallback = (int (__cdecl *)(unsigned char,void *,void *))ptr;
	m_hThreadTerm = CreateEvent( 0,TRUE,0,0);
	m_hThreadStarted = CreateEvent( 0,TRUE,0,0);
	m_hThreadStoppedEvent = CreateEvent( NULL, FALSE, FALSE, "ThreadStoppedEvent");
	ResetEvent(m_hThreadStoppedEvent);

	// Initialize our field ptrs...
	fielddata[0] = (void *)&fields.samplefloat;
	fielddata[1] = (void *)&fields.sampleint;
	fielddata[2] = (void *)&fields.samplestring[0];

	// make a thread for our driver to run in...
	threadid = 0;
	hThread = CreateThread(NULL, NULL, ThreadProc, NULL, NULL, (void *)&threadid);
	if (!hThread)
	{
		CloseHandle(m_hThreadTerm);
		CloseHandle(m_hThreadStarted);
		CloseHandle(m_hThreadStoppedEvent);
		m_hThreadTerm = NULL;
		m_hThreadStarted = NULL;
		m_hThreadStoppedEvent = NULL;
		return 0;
	}

	return 1;
}

// Shutdown
// This API is called by WinLog to shut the driver down, you should disconnect
// from the device and clean up any allocated resources here. 
// If the shutdown is successfull, return a 1, otherwise, return 0.
SAMPLE_API int Shutdown(void)
{

	if (m_hThreadTerm)
		SetEvent(m_hThreadTerm);

	if (m_hThreadStoppedEvent)
	{
		// wait here for up to 5 seconds for the object thread to stop...
		if (WaitForSingleObject(m_hThreadStoppedEvent, 5000) != WAIT_OBJECT_0)
			return FALSE;
	}

	if (m_hThreadTerm)
	{
		CloseHandle(m_hThreadTerm);
		m_hThreadTerm = NULL;
	}

	if (m_hThreadStarted)
	{
		CloseHandle(m_hThreadStarted);
		m_hThreadStarted = NULL;
	}

	if (m_hThreadStoppedEvent)
	{
		CloseHandle(m_hThreadStoppedEvent);
		m_hThreadStoppedEvent = NULL;
	}

	return 1;
}

// GetLastDriverError
// WinLog calls this function to get a detailed explaination of the last failure, simply
// return a string detailing it here!
SAMPLE_API char *GetLastDriverError(void)
{
	return "";
}

// GetFieldName
// WinLog Calls this api to get the name of the field specified in "FieldNum", if you're
// leaving the structures defined above intact, you probably won't need to change this field.
SAMPLE_API char *GetFieldName(int fieldnum)
{
	if (fieldnum >= NUM_FIELDS)
		return NULL;
	
	return fieldnames[fieldnum];
}

// GetFieldFormat
// WinLog Calls this api to get the name of the field format specified in "FieldNum", if you're
// leaving the structures defined above intact, you probably won't need to change this field.
SAMPLE_API char GetFieldFormat(int fieldnum)
{

	if (fieldnum >= NUM_FIELDS)
		return 0;
	
	return fieldformats[fieldnum];
}

// GetFieldData
// This API is called by Winlog to get the data for a specific field after the driver notifies
// WinLog that data is ready (by calling the notifycallback function with a 0). Simply
// return a pointer to the field value. if you're
// leaving the structures defined above intact, you probably won't need to change this field.
SAMPLE_API void *GetFieldData(int fieldnum)
{
	if (fieldnum >= NUM_FIELDS)
		return NULL;

	return fielddata[fieldnum];	
}


// GetDeviceName
// This API is called by WinLog to determine the name of your device, simply return the
// name of the device.
static char namebuffer[128];
SAMPLE_API char *GetDeviceName(void)
{
	strcpy(namebuffer, "Sample Winlog Driver");
	return namebuffer;
}

// GetDriverVersion
// This API is called by WinLog to determine the version of your driver, return this
// as an integer value.
SAMPLE_API int GetDriverVersion(void)
{
	return 1;
}

// Configure
// This API is called by Winlog to have the driver display its configuration dialog, you
// should not reconfigure the device here, just display the configuration, and update the
// configuration structure, WinLog will reconfigure the device by calling initialize again
// later
SAMPLE_API int Configure(HWND hParent)
{
	if (ShowConfigDialog(hParent) == IDOK)
	{
		// don't actually reconfigure the device here,
		// just configure the structures, the front end will re-initialize the device with the new settings.
		return 1;
	}
	else
		return 0;
}

// GetDeviceFunction
// WinLog Calls this api to get the name of the device function specified in "functionnum".
// You should return the user readable name of the function requested, or NULL if the
// function is not a valid function number.
SAMPLE_API char *GetDeviceFunction(int functionnum)
{
	switch (functionnum)
	{
	case 0:
		{
			return "reset sample values";
		}
		break;
	}

	return NULL;
}

// ExecuteDeviceFunction
// WinLog calls this APT to executes a device specific functions.
// A return value of -1 should be returned if the function is
// not supported. The function numbers should match the numbers returned
// in the description
// of the functions returned in the GetDeviceFunction API.
SAMPLE_API int ExecuteDeviceFunction(LPSTR functionname)
{
	if (!strcmp(functionname, "reset sample values"))
	{
		fields.samplefloat = 0;
		fields.sampleint = 0;
		strcpy(fields.samplestring, "test0");
		return 1;
	}

	return -1;
}







