/*

 A Sample WinLog device driver .dll
 Created by Justin Luton - 5/24/09 DevTechnics (http://www.devtechnics.com)

PUBLIC SOFTWARE 
We have placed this source code in the public domain. You can use, modify, and distribute
the source code and executable programs based on the source code. 

DISCLAIMER OF WARRANTY 
This source code is provided "as is" and without warranties as to performance or merchantability.
The author and/or distributors of this source code may have made statements about this source code.
Any such statements do not constitute warranties and shall not be relied on by the user in deciding
whether to use this source code. 

This source code is provided without any express or implied warranties whatsoever. Because of the
diversity of conditions and hardware under which this source code may be used, no warranty of fitness
for a particular purpose is offered. The user is advised to test the source code thoroughly before relying
on it. The user must assume the entire risk of using the source code. 

*/

#include "stdafx.h"
#include "sample.h"
#include "stdio.h"
#include "stdlib.h"
#include "math.h"








// *****************************************************************************************************
// Declarations of things that you'll most likely be changing
// *****************************************************************************************************

// The name of your device driver
#define DEVICE_DRIVER_NAME "WinLog Sample Serial Driver"

// the version of your driver (must be an integer value)
#define DRIVER_VERSION 1

// The number of fields that we'll be sending back to WinLog, this is normally a count
// of the variables defined in the "FIELD_DATA" structure above. Change this to match
// the number of fields defined above.
#define NUM_FIELDS 4

// The display names for each of the fields that will be sent back and displayed to the user.
// Change this to describe your fields
char *fieldnames[]=
{
	"rxbytes",
	"sample int",
	"sample float",
	"sample string"
};

// the format of each of the fields that will be sent back to WinLog:
// 'f' - floating point field 
// 'd' - integer field
// 's' - string field (a char buffer)
// Change this to describe the data type for each of your fields.
char *fieldformats = "ddfs";

// A structure to store our field variables, change this structure
// to include the list of your devices fields, these field types
// should be only float, int, or char. They should be in the same order
// as the fieldnames list above!!! They should also be the same types
// as specified in the fieldformats buffer above!!!

typedef struct
{
	int rxbytes;
	int sampleint;
	float samplefloat;
	char samplestring[32];
} FIELD_DATA;













// *****************************************************************************************************
// Declartions of things you probably won't be changing
// *****************************************************************************************************

// Handle to our instance
HANDLE hInst = NULL;

// A variable to store our configuration parameters in, normally,
// you won't need to change this declaration! The configuration
// structure is defined in sample.h
DEVICE_CONFIG config;

char drivererrorstring[128];

void OutputDebugMessage(LPSTR msg);

// A global handle to our com port
HANDLE hCom = NULL;


// Some object handles to notify our driver thread of events
HANDLE m_hThreadTerm = NULL;
HANDLE m_hThreadStarted = NULL;
HANDLE m_hThreadStoppedEvent = NULL;

// This is simply an optimization so that the driver can quickly lookup a field when WinLog requests
// it, you'll see below that we set pointers to the fields defined in the fields structure above when
// we initialize below.
void *fielddata[NUM_FIELDS];

// prototype for the callback to Winlog, initialized below in the Initialize function, you shouldn't
// need to change this!
int (__cdecl *notifycallback)(unsigned char notification, void *hint, void *hint2) = NULL;
void *notifyhint = NULL;

// Some COM Port stuff
DCB dcb;
unsigned char COMBuffer[1024];
#define RXBUFFSIZE 1024
unsigned char RXBuffer[RXBUFFSIZE];

FIELD_DATA fields;
int notifieddisconnect = 0;
int AbortSerialProc = 0;
unsigned long lastrxtime = 0;
int TotalBytesReceived = 0;
int BytesInBuff = 0;

// Forward functions declarations
DWORD WINAPI NewThreadProc(LPVOID lpParameter);
void ReceivedData(int BytesReceived);
int OpenCOMPort(int portnum);
int SendData(const char* data,DWORD dwSize);
















// *****************************************************************************************************
// Below are functions that you'll most likely be changing to support your device....
// *****************************************************************************************************


/*
// Initialize
// This function is called by winlog to initialize the driver, communication with the device
// should be attempted here. Also, the device can be configured here, since it is guaranteed
// that SetConfig will be called prior to this, so your configuration data will be valid! 
// The function should return 1 if the device was sucessfully initialized, otherwise, return
// 0.

// The following parameters are passed to initialize:
HWND parent - The parent window that is initializing the driver
void *ptr - A pointer to the callback function that you can call to notify winlog of driver
            events. (See below for notification values)
void *hint - A value that should always be passed back as the NotifyCallback hint parameter.


NotifyCallback spec:
The notifycallback function informs WinLog of driver events, The parameters to the function
are:

NotifyCallback (unsigned char notification, void *hint, void *hint2);

unsigned char notification - This is the type of notification that you are informing WinLog
of, it can be one of the following:

// A notify callback value of 0 represents a sample is ready to be retrieved...
	notifycallback(0, notifyhint, 0);


// A notify callback value of 1 requests the front end to open a log file for us to dump data to, without
// updating gauges during the download. This is usually done to download stored log data to a file. If there
// is a failure to open the logfile, the function will return 0, otherwise, non-zero
if (!notifycallback(1, notifyhint, 0))
{
	return 0;
}

// A notify callback value of 2 requests the front end to close the currently open log file.
notifycallback(2, notifyhint, 0);

// a notify callback value of 3 will send a debug message to winlog.
if (notifycallback)
	notifycallback(3, notifyhint, msg);

void *hint - This value should always be the "hint" parameter that was passed in to the
             Initialize function.

void *hint2 - Notification specific. For example, the "Debug Message" notification uses
              this parameter to pass a Debug String to winlog.

*/
SAMPLE_API int Initialize(HWND parent, void *ptr, void *hint)
{
	DWORD threadid = 0;
	HANDLE hThread;

	// Setup our callback routines into WinLog, this piece of code should
	// rarely change...
	notifyhint = hint;
	notifycallback = (int (__cdecl *)(unsigned char,void *,void *))ptr;

	// Initialize our field ptrs, normally, you'll just add to this list, pointing at
	// each successive element in the fielddata structure...
	fielddata[0] = (void *)&fields.rxbytes;
	fielddata[1] = (void *)&fields.samplefloat;
	fielddata[2] = (void *)&fields.sampleint;
	fielddata[3] = (void *)&fields.samplestring[0];

	// Set some defaults, this is driver dependant, you should set all of your defaults here!
	config.comport = 1;
	drivererrorstring[0]=NULL;

	// attemp to open our com port...
	if (!OpenCOMPort(config.comport))
		return 0;

	// create some events that we use to notify our driver thread to stop...
	m_hThreadTerm = CreateEvent( 0,TRUE,0,0);
	m_hThreadStarted = CreateEvent( 0,TRUE,0,0);

	AbortSerialProc = 0;
	threadid = 0;

	// And create our driver thread, execution of the driver will now switch to the
	// "NewThreadProc" function.
	hThread = CreateThread(NULL, NULL, NewThreadProc, NULL, NULL, (void *)&threadid);
	if (!hThread)
	{
		sprintf(drivererrorstring, "error creating com thread");
		OutputDebugMessage(drivererrorstring);
		CloseHandle(m_hThreadTerm);
		CloseHandle(m_hThreadStarted);
		m_hThreadTerm = NULL;
		m_hThreadStarted = NULL;

		return 0;
	}
	else
	{
		m_hThreadStoppedEvent = CreateEvent( 
	     NULL,         // no security attributes
	     FALSE,         // manual-reset event
	     FALSE,         // initial state is signaled
	     "ThreadStoppedEvent"  // object name
	     ); 

		ResetEvent(m_hThreadStoppedEvent);
	}

	return 1;
}

// Shutdown
// This API is called by WinLog to shut the driver down, you should disconnect
// from the device and clean up any allocated resources here. 
// If the shutdown is successfull, return a 1, otherwise, return 0.
SAMPLE_API int Shutdown(void)
{

	OutputDebugMessage("Shutting Down");

	// Tell our driver thread to stop...
	if (m_hThreadTerm)
		SetEvent(m_hThreadTerm);

	AbortSerialProc = 1;


	if (m_hThreadStoppedEvent)
	{
		// wait here for up to 5 seconds for the object thread to stop...
		if (WaitForSingleObject(m_hThreadStoppedEvent, 5000) != WAIT_OBJECT_0)
			return FALSE;
	}

	if (hCom)
	{
		CloseHandle(hCom);
		hCom = NULL;
	}

	if (m_hThreadTerm)
	{
		CloseHandle(m_hThreadTerm);
		m_hThreadTerm = NULL;
	}

	if (m_hThreadStarted)
	{
		CloseHandle(m_hThreadStarted);
		m_hThreadStarted = NULL;
	}

	if (m_hThreadStoppedEvent)
	{
		CloseHandle(m_hThreadStoppedEvent);
		m_hThreadStoppedEvent = NULL;
	}

	OutputDebugMessage("Shutdown Complete!");

	return 1;
}


// Configure
// This API is called by Winlog to have the driver display its configuration dialog, you
// should not reconfigure the device here, just display the configuration, and update the
// configuration structure, WinLog will reconfigure the device by calling initialize again
// later
SAMPLE_API int Configure(HWND hParent)
{
	// You should put your configuration dialog box in here.
	// When the user has completed configuration, copy the settings back
	// to the 'config' structure
	return 1;
}

// This is the main thread procedure for our driver, you'll want to do the specifics
// of communucating with your device in here!
DWORD WINAPI NewThreadProc(LPVOID lpParameter)
{
    HANDLE arHandles[2];
  	OVERLAPPED ov;
	DWORD BytesReceived = 0;
    DWORD dwWait;
	DWORD dwEventMask;
	BOOL abRet;
	char outbuffer[10];

    memset(&ov,0,sizeof(ov));
    ov.hEvent = CreateEvent( 0,TRUE,0,0);
    arHandles[0] = m_hThreadTerm;

	// Notify the creator that we've started our thread...
    SetEvent(m_hThreadStarted);

	// In our example, we'll send a byte to our device to request it to start sending us data, 
	// your device will definitely be different!
	outbuffer[0] = '?';
	SendData(outbuffer, 1);

	// And we'll now run our loop, waiting for data from the COM port,
	// and watching for a terminate thread event...
    while (1)	
    {
        
		// Wait for a COM Event to happen, and detect errors
        abRet = WaitCommEvent(hCom,&dwEventMask, &ov) ;
        if ( !abRet )
        {
			// Some Problem?
			DWORD error = GetLastError ();
			if ((error != ERROR_IO_PENDING) && (error != 87))
			{
				CloseHandle(ov.hEvent);
				if (m_hThreadStoppedEvent)
					SetEvent(m_hThreadStoppedEvent);

				AbortSerialProc = 1;
				notifieddisconnect = 1;
				notifycallback(4, notifyhint, 0);

				return 0;
			}
		}
        
		// Now wait for the COM event, a TERM event, or timeout in 1 second...
        arHandles[1] = ov.hEvent ;
        
        dwWait = WaitForMultipleObjects (2,arHandles,FALSE,1000);
	
        switch ( dwWait )
        {
			case WAIT_TIMEOUT:
			{
				// you may just want to do something like re-request
				// data from your device here, but in this example,
				// we assume that if we haven't received any data in a
				// second, that the device is GONE!

				if ((notifycallback && !notifieddisconnect))
				{
					AbortSerialProc = 1;
					notifieddisconnect = 1;

					// tell winlog that we've had a 'disconnect'
					notifycallback(4, notifyhint, 0);
				}
			}
			break;
		case WAIT_OBJECT_0:
            {
				// We've received a TERMINATE request from winlog, cleanup, and exit the thread
				CloseHandle(ov.hEvent);

				// notify the outside world that we've stopped the thread
				if (m_hThreadStoppedEvent)
					SetEvent(m_hThreadStoppedEvent);

				return 0;
            }
            break;
        case WAIT_OBJECT_0 + 1:
            {
				// An event has happened on the COM port...
                DWORD dwMask;
                if (GetCommMask(hCom,&dwMask) )
                {
				   
				   if ( dwMask & EV_RXCHAR)
				   {
						//we've received data!
						OVERLAPPED ovRead;
						memset(&ovRead,0,sizeof(ovRead));
						ovRead.hEvent = CreateEvent( 0,TRUE,0,0);

						// Read data while it's available on the COM port...
						do
						{
							
							int result = ReadFile(hCom, (LPVOID)COMBuffer, 1024, (LPDWORD)&BytesReceived, (LPOVERLAPPED)&ovRead);
							if (!result)
							{
								DWORD err = GetLastError();
							}
							TotalBytesReceived += BytesReceived;
							
							if (BytesReceived)
							{
								lastrxtime = GetTickCount();

								// Process the data...
								ReceivedData(BytesReceived);
							}

						} while (BytesReceived && !AbortSerialProc);

						CloseHandle(ovRead.hEvent);

				   }

                   ResetEvent ( ov.hEvent );
                   continue;
                }
            }
        }//end switch
    }//end while

	CloseHandle(ov.hEvent);

	if (m_hThreadStoppedEvent)
		SetEvent(m_hThreadStoppedEvent);


	return 0;
}


// This function will be called whenever data has been received on the COM port...
void ReceivedData(int BytesReceived)
{
	// let's update our fields with the new byte count. This is normally where
	// you would parse out your sample data, and transfer it to the fields structure.
	// Note that in this function, you may not receive all of the packet data, and will
	// need to buffer up a full packet before processing it!!!
	fields.rxbytes += BytesReceived;
	fields.samplefloat = 0.0f;
	fields.sampleint = 0;
	fields.samplestring[0] = NULL;

	// We'll assume for this little sample, that we now have a full sample ready,
	// let's now let WinLog know about it...
	// A notify callback value of 0 represents a sample is ready to be retrieved...
	if (notifycallback)
		notifycallback(0, notifyhint, 0);

}


















// *****************************************************************************************************
// Things below this point are helper functions for our sample driver, and rarely modified
// WinLog API functions, probably things you don't care about!
// *****************************************************************************************************

// The main entry point to our driver
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{

	hInst = hModule;
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

// A helper function to open a com port, you may want to modify this to set the baud rate, etc...
int OpenCOMPort(int portnum)
{
	char buffer[255];
	BOOL fSuccess;
	COMMTIMEOUTS Timeouts;

	notifieddisconnect = 0;
	TotalBytesReceived = 0;

	sprintf(buffer, "\\\\.\\COM%d", portnum);

	hCom = CreateFile(buffer, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING , FILE_FLAG_OVERLAPPED, NULL);

	if (hCom == INVALID_HANDLE_VALUE)
	{
		sprintf(buffer, "COM%d", portnum);
		hCom = CreateFile(buffer, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING ,FILE_FLAG_OVERLAPPED, NULL);
		if (hCom == INVALID_HANDLE_VALUE)
		{
			sprintf(drivererrorstring, "error opening port");
			OutputDebugMessage(drivererrorstring);
			hCom = NULL;
			return FALSE;
		}
	}

	fSuccess = GetCommState(hCom, &dcb);
	if (!fSuccess)
	{
		sprintf(drivererrorstring, "error getting com state");
		OutputDebugMessage(drivererrorstring);
		CloseHandle(hCom);
		hCom = NULL;
		return FALSE;
	}

	dcb.BaudRate = 19200;
	dcb.ByteSize = 8;
	dcb.StopBits = ONESTOPBIT;
	dcb.Parity = NOPARITY;

	dcb.fOutxCtsFlow = 0;
    dcb.fOutxDsrFlow = 0;
    dcb.fDtrControl = 0;
    dcb.fDsrSensitivity=0;
    dcb.fTXContinueOnXoff=0;
    dcb.fOutX=0;
    dcb.fInX=0;
    dcb.fErrorChar=0;
    dcb.fNull=0;
    dcb.fRtsControl=0;
    dcb.fAbortOnError=0;

	fSuccess = SetCommState(hCom, &dcb);

	if (!fSuccess)
	{
		sprintf(drivererrorstring, "error setting com state");
		OutputDebugMessage(drivererrorstring);
		CloseHandle(hCom);
		hCom = NULL;
		return FALSE;
	}

	Timeouts.ReadIntervalTimeout = MAXDWORD; 
    //Timeouts.ReadIntervalTimeout = 10; 
	Timeouts.ReadTotalTimeoutMultiplier = 0; 
    Timeouts.ReadTotalTimeoutConstant = 0; 
    Timeouts.WriteTotalTimeoutMultiplier = 0; 
    Timeouts.WriteTotalTimeoutConstant = 100; 

	fSuccess = SetCommTimeouts(hCom, &Timeouts);

	if (!fSuccess)
	{
		sprintf(drivererrorstring, "error setting com timeouts");
		OutputDebugMessage(drivererrorstring);
		CloseHandle(hCom);
		hCom = NULL;
		return FALSE;
	}

	fSuccess = SetCommMask( hCom, EV_TXEMPTY | EV_RXCHAR);

	if (!fSuccess)
	{
		sprintf(drivererrorstring, "error setting com mask");
		OutputDebugMessage(drivererrorstring);
		CloseHandle(hCom);
		hCom = NULL;
		return FALSE;
	}

	return TRUE;
}

// a helper to clear our input buffer
void ClearRxBuffer(void)
{
	BytesInBuff = 0;
}

// a helper to get the number of bytes in the input buffer
int GetRXCount(void)
{
	return BytesInBuff;
}

// a helper to return the input buffer
LPBYTE GetRXBuffer(int *count)
{
	if (count)
		*count = BytesInBuff;

	if (!BytesInBuff)
		return NULL;

	return RXBuffer;
}

// a helper to send data out of the COM port...
int SendData(const char* data,DWORD dwSize)
{
	int iRet = 0 ;
	OVERLAPPED ov;
	DWORD dwBytesWritten = 0;

	if (!hCom)
		return FALSE;

	memset(&ov,0,sizeof(ov));
	ov.hEvent = CreateEvent( 0,TRUE,0,0);
	{
		iRet = WriteFile (hCom,data,dwSize,&dwBytesWritten  ,&ov);
		if ( iRet == 0 )
		{
			WaitForSingleObject(ov.hEvent ,INFINITE);
		}
 
	}

	CloseHandle(ov.hEvent);
	
	return TRUE;
}

// a helper to send a debug message to WinLog. (Will be stored in the debuglog.txt file)
void OutputDebugMessage(LPSTR msg)
{
	if (notifycallback)
		notifycallback(3, notifyhint, msg);

	return;
}

// GetDriverVersion
// Called by WinLog to determine the version of the driver.
SAMPLE_API int GetDriverVersion(void)
{
	return DRIVER_VERSION;
}

// GetDeviceFunction
// WinLog Calls this api to get the name of the device function specified in "functionnum".
// You should return the user readable name of the function requested, or NULL if the
// function is not a valid function number.
SAMPLE_API char *GetDeviceFunction(int functionnum)
{
	switch (functionnum)
	{
	case 0:
		{
			return "sample function";
		}
		break;
	}

	return NULL;
}

// ExecuteDeviceFunction
// WinLog calls this API to executes a device specific functions.
// A return value of -1 should be returned if the function is
// not supported. The function numbers should match the numbers returned
// in the description
// of the functions returned in the GetDeviceFunction API.
SAMPLE_API int ExecuteDeviceFunction(LPSTR functionname)
{
	if (!strcmp(functionname, "sample function"))
	{
		// our dummy function simply resets the rxbytes field for now...
		fields.rxbytes = 0;
		return 1;
	}

	return -1;
}

// SetConfig
// This API is called by Winlog to notify the driver of it's configuration, no actual 
// reconfiguration of the device should be done here, instead, just copy the configuration
// to your local structure. Usually, this code won't need to be changed.
SAMPLE_API void SetConfig(void *pConfig)
{
	memcpy(&config, pConfig, sizeof(DEVICE_CONFIG));
}

// GetConfig
// This API is called by Winlog to get the current configuration of the driver.
// Return a pointer to the configuration data.
// Usually, this code won't need to be changed.
SAMPLE_API void *GetConfig(void)
{
	return (void*)&config;
}

// GetConfigSize
// This API is called by winlog to determine the size of the configuration data that the
// driver needs to be stored.
// Return the size of the configuration data in bytes.
// Usually, this code won't need to be changed.
SAMPLE_API int GetConfigSize(void)
{
	return sizeof(DEVICE_CONFIG);
}

// GetLastDriverError
// WinLog calls this function to get a detailed explaination of the last failure, simply
// return a string detailing it here!
SAMPLE_API char *GetLastDriverError(void)
{
	return drivererrorstring;
}


// GetFieldName
// WinLog Calls this api to get the name of the field specified in "FieldNum", if you're
// leaving the structures defined above intact, you probably won't need to change this field.
SAMPLE_API char *GetFieldName(int fieldnum)
{
	if (fieldnum >= NUM_FIELDS)
		return NULL;
	
	return fieldnames[fieldnum];
}

// GetFieldFormat
// WinLog Calls this api to get the name of the field format specified in "FieldNum", if you're
// leaving the structures defined above intact, you probably won't need to change this field.
SAMPLE_API char GetFieldFormat(int fieldnum)
{

	if (fieldnum >= NUM_FIELDS)
		return 0;
	
	return fieldformats[fieldnum];
}

// GetDeviceName
// This API is called by WinLog to determine the name of your device, simply return the
// name of the device.
static char namebuffer[128];
SAMPLE_API char *GetDeviceName(void)
{
	strcpy(namebuffer, DEVICE_DRIVER_NAME);
	return namebuffer;
}


// GetFieldData
// This API is called by Winlog to get the data for a specific field after the driver notifies
// WinLog that data is ready (by calling the notifycallback function with a 0). Simply
// return a pointer to the field value. if you're
// leaving the structures defined above intact, you probably won't need to change this field.

SAMPLE_API void *GetFieldData(int fieldnum)
{
	if (fieldnum >= NUM_FIELDS)
		return NULL;

	return fielddata[fieldnum];	
}

// GetDefaultLogState
// This API is called by Winlog to determine if a field is, by default logged. Return a zero
// if you prefer that a field is not logged, generally, you'll leave this value to be a '1'
SAMPLE_API int GetDefaultLogState(int fieldnum)
{
	return 1;

}

